%Creates the multiple models, i.e. the inverse, forward and responsibility
%predictor. The execution function handle must point to the function that
%acts on the environment, i.e. the issuing of the motor commands. In
%addition, the execution function handle must return the state of the
%environment *after* the action has been performed. When called with the
%empty matrix, the execute_function_handle should return the state of the
%environment/initial state of the environment (i.e. it can be used as a
%resetting method also).
%
%mm = multiple_models(number_of_mms, inv_size, forw_size, rp_size, execution_function_handle)
%
%Inputs:
%
%number_of_mms: The number of paired inverse and forward models.
%
%inv_size: A matrix of the form [input hidden output] that determines the
%size of the recurrent neural network used for the inverse model.
%
%forw_size: A matrix of the form [input hidden output] that determines the
%size of the recurrent neural network used for the forward model.
%
%rp_size: A matrix of the form [input hidden output] that determines the
%size of the recurrent neural network used for the responsibilty predictor.
%
%execution_function_handle: A function handle where the actual motor
%commands will be performed. If there exists a function called
%simulator_execute, it can be passed as an argument like this:
%@simulator_execute.
%
%K: The feedback error motor gain.
%
%output_gain: The output gain.
%
%learning_rate: The learning rate of the neural networks.
%
%sigma: The sigma value used in the likelihood function.
%
%error_proportion: How much the winning module should be biased when the
%normalization of the lambda values occur. After the primary normalization,
%the winning lambda value is multiplied with error_proportion, and the
%lambda values are normalized again. The winning lambda value will now
%receive more of its error signal than the other modules.
%
%Outputs:
%
%mm: A structure holding the neural networks for the inverse model, forward
%model, responsibility predictor, the execution function, the feedback
%error gain, the output gain, the learning rate, the sigma used in the
%likelihood function and the error proportion value. These latter variables
%are set manually in this file, since they were considered too numerous to
%include as input arguments.

function mm = multiple_models(number_of_mms, inv_size, forw_size, rp_size, execution_function_handle, K, output_gain, learning_rate, sigma, error_proportion)

%Seed the random number generator
rand('state',sum(100*clock));

%The execution function that sends the motor commands to the simulator.
mm.execute = execution_function_handle;

%The gain used for feedback error learning.
mm.K = K;

%The gain for the output, since the neural networks only output in the
%range [0,1] which is scaled to [-1,1].
mm.output_gain = output_gain;

%The sigma value used in the likelihood estimation.
mm.sigma = sigma;

%This determines how much more of the error signal the winning module will
%receive. All the lambda values are normalized before computation of the
%motor commands. After the motor commands are computed, the winning lambda
%value will be multiplied with this value, and then the lambda values will
%be normalized again. 
mm.error_proportion = error_proportion;

%If printing activation levels should be on or off
mm.verbose = false;

for i=1:number_of_mms
    mm(i).inverse = rnn(inv_size, learning_rate);
    mm(i).forward = rnn(forw_size, learning_rate);
    mm(i).rp = rnn(rp_size, learning_rate);
end