%Plots the attractor of the hidden neural activation of the multiple model.
%This is done by running the simulator with the multiple model and saving the
%neural activation as we go. Uncertainty: which plots to do. 1 & 2 and 2 &
%3 or just 1-2 2-4 4-6? Initially, do all for the entire period for both
%inverse and forward models. Possible extension: for specific periods of
%time also. Might also require to remove the initial transient, i.e. when
%the memory is not there. Could be resolved by running the simulator
%several times through the desired state, but that has never been done for
%the multiple model before. 

function handles = plot_attractors(mm, mm_error_log, run_simulator, epoch)

if nargin < 4
    epoch = length(mm_error_log);
end

filtered = false;

[ states, mms ] = run_simulator(mm, mm(1).desired_state, mm(1).context);


%Filter the hidden neural activation based on
%the winning module. I.e. the attractor plot will show when the module
%was winning.
%Vertically concatenate the lambda activations for the different
%modules for the last epoch.

if filtered
    lambda_activation = [];
    for j=1:length(mm_error_log(epoch).err)
        lambda_activation = [ lambda_activation ; mm_error_log(epoch).err(j).lambda ];
    end
    
    %Sort the lambda activations, and keep the indices of the winning
    %modules. The indices will point to which module was the winning module
    %in the previous run.
    [Y, winning_modules] = sort(lambda_activation, 1, 'descend');
end

%The handles matrix. Initialized to the empty matrix, so we can concatenate
%them.
handles = [];


%The number of columns in the output will depend on if the filtering is to
%be shown or not.
plot_cols = 2;
if filtered
    plot_cols = 4;
end


%For each of the modules, plot the neural activation of the hidden layer of
%the forward and inverse models. 
for i=1:length(mm)
    handles = [ handles figure('Name', [ 'Attractors of module ' num2str(i) ]) ];
    %The number of plots depends on the 
    %size of the hidden layer. It is the same for the forward and
    %inverse module.
    num_plots = mm(1).inverse.size(2) - 1;
    
    %Collect all the activation levels of the forward and inverse module.
    hidden_activation_forward = zeros(mm(1).forward.size(2), length(mms));
    hidden_activation_inverse = zeros(mm(1).inverse.size(2), length(mms));
    for j=1:length(mms)
        hidden_activation_forward(:,j) = mms{j}(i).forward.L{2};
        hidden_activation_inverse(:,j) = mms{j}(i).inverse.L{2};
    end

    
    %Do the filtering
    
    if filtered
        hidden_activation_forward_filtered = [];
        hidden_activation_inverse_filtered = [];
        
        %Run through the matrix holding the winning modules. The number in the first
        %row of each column corresponds to the module that was
        %the best module at that timestep. The outer loop is over the number
        %of modules, so if the current module was actually the winner node,
        %we can save the neural activation and plot it.
        for j=1:length(winning_modules)
            if winning_modules(1,j) == i
                hidden_activation_forward_filtered = [hidden_activation_forward_filtered hidden_activation_forward(:,j) ];
                hidden_activation_inverse_filtered = [ hidden_activation_inverse_filtered hidden_activation_inverse(:,j) ];
            end
        end
    end

    %Plot counter.
    plot_counter = 1;
    %Plot the attractors.
    for j=1:num_plots

        %Plot the forward model attractors.

        subplot(num_plots, plot_cols, plot_counter);

        plot(hidden_activation_forward(j,:), hidden_activation_forward(j+1,:), 'b');
        
        axis square
        
        if j == 1
            title('Forward model attractors');
        end

        xlim([0 1 ]);
        ylim([0 1 ]);
        
        xlabel(['Context node ' num2str(j) ]);
        ylabel(['Context node ' num2str(j+1) ]);
        
        plot_counter = plot_counter + 1;

        
        
        
        
        
        
        
        if filtered
        
            %Plot the FILTERED forward model attractors.
            
            subplot(num_plots, plot_cols, plot_counter);
            
            plot(hidden_activation_forward_filtered(j,:), hidden_activation_forward_filtered(j+1,:), 'b');
            
            axis square
            
            if j == 1
                title('Filtered forward model attractors');
            end
            
            xlim([0 1 ]);
            ylim([0 1 ]);
        
            xlabel(['Context node ' num2str(j) ]);
            ylabel(['Context node ' num2str(j+1) ]);
            
            plot_counter = plot_counter + 1;
            
        end
        
        
        
        
        
        %Plot the inverse model attractors.
        
        subplot(num_plots, plot_cols, plot_counter);
        
        plot(hidden_activation_inverse(j,:), hidden_activation_inverse(j+1,:), 'r');
        
        axis square

        if j == 1
            title('Inverse model attractors');
        end
                
        xlabel(['Context node ' num2str(j) ]);
        ylabel(['Context node ' num2str(j+1) ]);
        
        xlim([0 1 ]);
        ylim([0 1 ]);
        
        plot_counter = plot_counter + 1;

        
        
        if filtered

            %Plot the FILTERED inverse model attractors.
            
            subplot(num_plots, plot_cols, plot_counter);
        
            plot(hidden_activation_inverse_filtered(j,:), hidden_activation_inverse_filtered(j+1,:), 'r');
            
            axis square
            
            if j == 1
                title('Filtered inverse model attractors');
            end
            
            xlabel(['Context node ' num2str(j) ]);
            ylabel(['Context node ' num2str(j+1) ]);
            
            xlim([0 1 ]);
            ylim([0 1 ]);
            
            plot_counter = plot_counter + 1;
        end
        
    end
    
    %Position the plot
    set(handles(i), 'Position', [ 1 1 plot_cols*170 num_plots*170 ], 'PaperPositionMode', 'auto');
    

end       

