%Plots the entire run of summed forward errors for each of the modules,
%the summation of the feedback error motor command, the mean ratio of
%feedback error command compared to actual motor command and how many
%transitions there has been between the winning modules.
%
%[h, occurences_of_transitions, performers] = plot_run(error_log, desired_state)
%
%Inputs:
%
%error_log: This is the error log for the entire run, a structure holding
%several error logs for each epoch. Normally referenced like this:
%hybrid_mosaic_error_log.
%
%desired_state: This is the desired state that the performance of the
%system as a whole is measured against.
%
%Outputs:
%
%h: The handle to the figure.
%
%occurences_of_transitions: A vector holding the epochs where a change in
%the number of transitions occured.
%
%performers: The sorted list of performance of the epochs, i.e. the first
%element is the epoch that had the least error and so on.

function [h, occurences_of_transitions, performers] = plot_run(error_log, desired_state)

h = figure('Name', 'Plot of total run');

total_plot_size = length(error_log(1).err) + 4;

%To colorize each module. Now it will only work with 7 modules. If this is
%to be extended, do a simple [] concatenation, and repeat the colors. Or
%you could simply create some more colors from an RGB color table.
colors = [ 'b' 'r' 'm' 'c' 'g' 'y' 'k'];

%The i variable now holds the module number.
for i=1:length(error_log(1).err)
    
    summation = [];
    
    for j=1:length(error_log)
        summation = [ summation sum(error_log(j).err(i).forward) ];
    end
    
    subplot(total_plot_size, 1, i);
    plot(summation, [ colors(i) '--' ]);
    ylim( [ 0 max(summation) ]);
    
    xlabel([ 'Summation of the absolute forward error, module ' num2str(i) ]);
    
    %Now plot the summation of the forward error multiplied with its lambda
    %value, i.e. sum up the forward errors when it was actually the winning
    %module.
    
    hold on;
    
    summation = [];
    
    for j=1:length(error_log)
        summation = [ summation sum( error_log(j).err(i).forward .* error_log(j).err(i).lambda ) ];
    end
    
    subplot(total_plot_size, 1, i);
    plot(summation, colors(i));

    legend({'Error', 'Error * \lambda'});

    hold off;
end

%Plot the summation of the ufb commands
summation = [];
for j=1:length(error_log)
    summation = [ summation sum(error_log(j).err(1).inverse) ];
end

subplot(total_plot_size, 1, total_plot_size-3);
plot(summation, colors(end));
ylim( [ 0 max(summation) ] );

xlabel('Summation of the absolute feedback error motor command');

%Plot the mean of the ratio of the ufb to the motor commands, along with
%the standard deviation for each epoch.
avg = [];
stdev = [];
for j=1:length(error_log)
    avg = [ avg mean(error_log(j).err(1).ratio) ];
    stdev = [ stdev std(error_log(j).err(1).ratio) ];
end

subplot(total_plot_size, 1, total_plot_size-2);
plot(avg, colors(end));
ylim( [ 0 max(avg) ] );

hold on;

subplot(total_plot_size, 1, total_plot_size-2);
plot(avg + stdev);
plot(avg - stdev);

hold off;

xlabel('mean( ufb/motor command ) and standard deviation for each epoch');

%Plots the number of transitions between the modules. If there are two
%modules, and the "ruling" modules are 1-2-1, that counts as three
%transitions. 


transitions = ones(1, length(error_log));
occurences_of_transitions = [];

for i=1:length(error_log)
    %Vertically concatenate the lambda activations for the different
    %modules.
    lambda_activation = [];
    for j=1:length(error_log(i).err)
        lambda_activation = [ lambda_activation ; error_log(i).err(j).lambda ];
    end
    
    %Sort the lambda activations, and keep the indices of the winning
    %modules. The indices will point to which module was the winning module
    %in the previous run.
    [Y, indices] = sort(lambda_activation, 1, 'descend');
    
    %Run over all the winners, and for each winner (i.e. the value in the
    %first row of each column) see if it is the same winner as in the
    %previous column. If the winner is different, increase the number of
    %transitions for this epoch.
    for j=2:length(indices)
        if indices(1,j) ~= indices(1,j-1)
            transitions(i) = transitions(i) + 1;
        end
    end
    
    %Log when the transitions happened
    if i > 1 && transitions(i) ~= transitions(i-1)
        occurences_of_transitions = [ occurences_of_transitions i ];
    end
        
end

%Plot the development of transitions.
subplot(total_plot_size, 1, total_plot_size - 1);
plot(transitions, colors(end));
ylim( [ 0 max(transitions) + 1 ]);
xlabel('The number of transitions between winning modules for each epoch');    

%Calculate the total performance of the system, i.e. how much each epoch
%deviates from the target state and sum up the absolute error. Will make it
%easy to show exactly which epoch was the best. 
for i=1:length(error_log)
    %sum() is applied twice, because the first sum() sums up the columns,
    %the last sum sums up the resulting vector.
    total_performance(i) = sum(sum(abs(error_log(i).actual_state - desired_state)));
end

%Sort the performance array.
[X, performers] = sort(total_performance);

%Plot the total performance of the system.
subplot(total_plot_size, 1, total_plot_size);
plot(total_performance, colors(end));
xlabel('Summation of the absolute error for each epoch, i.e. sum(abs(desired state - actual state))');