%Training function for the recurrent neural network. The network is trained
%using backpropagation through time. Since this is a simple recurrent
%network (i.e. Elman network), this implies that the weights leading
%from the hidden layer to itself (e.g. the weights from the extended part
%of the input layer) are trained. 
%
%net = rnn_train(net, err, lambda)
%
%Inputs: 
%
%net: The neural network to be trained.
%
%err: The error, i.e. ( desired_output - actual_output)
%
%lambda: The responsibility signal of the network. The lambda value
%determines how much the network learns, cf. the temperature in simulated
%annealing. 
%
%Outputs:
%
%net: The neural network, trained on basis of the error input and the
%responsibility signal (lambda).

function net = rnn_train(net, err, lambda)

%If the lambda value is not specified, make lambda = 1 so it will not
%affect the learning.
if nargin < 3
    lambda = 1;
end

output_errors = net.L{3}.*(1 - net.L{3}).*err;

hidden_errors = net.L{2}.*(1 - net.L{2});

%Do not need to calculate the hidden error at the dummy node - it is always
%high anyway, and will not be propagated further back.
for i=1:length(hidden_errors)
    hidden_errors(i) = hidden_errors(i)*sum( output_errors .* net.w{2}(:,i) );
end

%Updating w2. Adding 1 for the input bias. Backpropagating the error from
%the output layer. The added one indicates that error will be propagated to
%the dummy node as well. 
for i=1:length(output_errors)
    net.w{2}(i,:) = net.w{2}(i,:) + net.lr .* lambda .* output_errors(i) .* [ net.L{2}' 1 ];
end

%Updating w1. Adding 1 for the input bias. Backpropagating error from the
%hidden layer.
for i=1:length(hidden_errors)
    net.w{1}(i,:) = net.w{1}(i,:) + net.lr .* lambda .* hidden_errors(i) .* [ net.L{1}' 1 ];
end