% Function used to run an experiment to test for catastrophic forgetting
% using a regular fully-connected feed-forward network trained with
% backpropagation.
%
%function [first,second,third] = run_cf_experiment(first, second, 
%         train_percentage, test_percentage, epochs, learning_rate,
%         hidden_size, criterion)
%
% Inputs:
% first: A data structure set with dataset to use for the first batch of
%   training and testing. This should be set in first.all.in and
%   first.all.out
% second A data structure containing the dataset to use for the second
%   batch of training. Similiar structure as for first
% train_percentage: How much of the dataset to use for training, in the
%   range [0-1]. This will be taken from the beginning of the dataset
%   vectors 
% test_percentage: How much of the dataset to use for training, in the
%   range [0-1]. This will be taken from the end of the datase vector. So
%   if train_percentage+test_percentage>1, there will be overlap in the
%   training and testing sets
% epochs: Integer determining the number of epochs the experiment will run
%   for
% learning_rate: The learning rate to use for the backprop training
% hidden_size: The number of nodes in the hidden layer
% criterion: A value k such that if t<=o+k and t>=o-k, the output o
%   classifies as the target t. This value is used when testing the network
%   to see if it classifies test patterns correctly.
%
% Outputs:
% All the outputs have had an additional field testlog added.  This
% contains a number denoting how big proportion of the test instances that
% were correctly classified in each run. These are padded so they are
% contains elements equal to the number of epochs even if the network
% converges earlier. first and second contains in addition fields X.train
% and X.test containing the training and testing instances used.
% first: Data for the first training batch performed
% second: Data for the second training batch performed
% third: Data for testing the first test batch during the second training.
function [first,second,third] = run_cf_experiment(first, second, ...
    train_percentage, test_percentage, epochs, learning_rate, ...
    hidden_size, criterion)

    %Calculate number of instances to use for training and testing
    train_num = round(size(first.all.in,1)*train_percentage);
    test_num = round(size(first.all.in,1)*test_percentage);

    %Create input and output sets properly structured for the neural
    %network
    first.train.in = permute(first.all.in(1:train_num,:), [2,1]);
    first.train.out = permute(first.all.out(1:train_num,:), [2,1]);     
    first.test.in = permute(first.all.in(end-test_num:end,:), [2 1]);
    first.test.out = permute(first.all.out(end-test_num:end,:), [2 1]);
    first.testlog = [];

    %Repeat for second training period
    train_num = round(size(second.all.in,1)*train_percentage);
    test_num = round(size(second.all.in,1)*test_percentage);
    second.train.in = permute(second.all.in(1:train_num,:), [2 1]);
    second.train.out = permute(second.all.out(1:train_num,:), [2 1]);
    second.test.in = permute(second.all.in(end-test_num:end,:), [2 1]);
    second.test.out = permute(second.all.out(end-test_num:end,:), [2 1]);
    second.testlog = [];

    %Keep a log of how well the performance on the first dataset develops
    %while learning the second
    third.testlog = [];

    %Create a new network with one hidden layer
    network_size = [size(first.train.in,1), hidden_size, ...
                    size(first.train.out,1)];
    net = rnn(network_size, learning_rate);

    %Start training first batch, storing the best net for later use
    bestnet = net;
    for i = 1 : epochs
        net = train_epoch(net, first.train.in, first.train.out);

        %Test the network and calculate percentage
        correct = test_ann(net, first.test.in, first.test.out, criterion);    
        percentage = correct/size(first.test.in,2);        

        %Store the best net so far
        if(percentage>max(first.testlog));     
            bestnet = net;
        end
        first.testlog(i) = percentage;
        
        %If 100% accuracy, stop training and set the testlog for the
        %remaining epochs to be 100% as well
        if(correct==size(first.test.out,2))
            first.testlog(i:epochs)=1;
            break       
        end
    end
     disp(sprintf('%s Best network classified %d%% correctly', ...
    'Training of first batch complete.', max(first.testlog)*100));

    % Now the network is trained as good as possible (or as good as we want
    % it). The next step is to train it to learn new information, this is
    % stored in second.all.in and second.all.out
    net = bestnet;

    for i = 1 : epochs
        net = train_epoch(net, second.train.in, second.train.out);

        correct = test_ann(net,second.test.in,second.test.out,criterion);    
        percentage = correct/size(second.test.in,2);       

        if(percentage > max(second.testlog))
            bestnet = net;
        end        
        second.testlog(i) = percentage;
        
        %test again with the first testset to see how it performs
        third.testlog(i) = test_ann(net, first.test.in, ...
          first.test.out, criterion)/size(first.test.in,2);

        if(correct==size(second.test.out,2))
            second.testlog(i+1:epochs)=1;
            third.testlog(i+1:epochs)=third.testlog(i);
            break       
        end
    end

    disp(sprintf('%s Best network classified %d%% correctly', ...
     'Training of second batch complete.', max(second.testlog)*100));
    disp(sprintf('%s of initial dataset is %d%% correctly', ...
     'Experiment complete. Final performance', third.testlog(end)*100));